/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *		http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef _BT_CORE_ADAPTER_H_
#define _BT_CORE_ADAPTER_H_

#include <sys/types.h>
#include <sys/wait.h>
#include <dlog.h>
#include <glib.h>

#ifdef __cplusplus
extern "C" {
#endif

#define CONNMAN_DBUS_NAME "net.connman"
#define CONNMAN_BLUETOOTH_TECHNOLOGY_PATH "/net/connman/technology/bluetooth"
#define CONNMAN_BLUETOTOH_TECHNOLOGY_INTERFACE "net.connman.Technology"

#define BT_CORE_NAME "org.projectx.bt_core"
#define BT_CORE_PATH "/org/projectx/bt_core"

#ifdef TIZEN_FEATURE_ACTD
#define BT_SYSTEMACT_HCI_LOGGER_START "bluetooth-hci-logger.service"
#define BT_SYSTEMACT_HCI_LOGGER_STOP "bluetooth-hci-logger-stop.service"
#define BT_SYSTEMACT_HCI_UP "bluetooth-hci-up.service"
#define BT_SYSTEMACT_HCI_DOWN "bluetooth-hci-down.service"
#define BT_SYSTEMACT_BLUEZ_START "bluez-start.service"
#define BT_SYSTEMACT_BLUETOOTH_SHARE_START "bluetooth-share-start.service"
#define BT_SYSTEMACT_RFKILL_UNBLOCK "bluetooth-rfkill-unblock.service"
#define BT_SYSTEMACT_RFKILL_BLOCK "bluetooth-rfkill-block.service"
#define BT_SYSTEMACT_STACK_UP "bluetooth-stack-up.service"
#define BT_SYSTEMACT_STACK_DOWN "bluetooth-stack-down.service"
#define BT_SYSTEMACT_STACK_DOWN_WITH_RADIO "bluetooth-stack-down-with-radio.service"
#else
#define BT_SYSTEMACT_HCI_LOGGER_START "/run/.hci_logger_start"
#define BT_SYSTEMACT_HCI_LOGGER_STOP "/run/.hci_logger_stop"
#define BT_SYSTEMACT_HCI_UP "/run/.hci_up"
#define BT_SYSTEMACT_HCI_DOWN "/run/.hci_down"
#define BT_SYSTEMACT_BLUEZ_START "/run/.bluez_start"
#define BT_SYSTEMACT_BLUETOOTH_SHARE_START "/run/.bluetooth_share_start"
#define BT_SYSTEMACT_RFKILL_UNBLOCK "/run/.rfkill_unblock"
#define BT_SYSTEMACT_RFKILL_BLOCK "/run/.rfkill_block"
#define BT_SYSTEMACT_STACK_UP "/run/.bt_stack_up"
#define BT_SYSTEMACT_STACK_DOWN "/run/.bt_stack_down"
#define BT_SYSTEMACT_STACK_DOWN_WITH_RADIO "/run/.bt_stack_down_with_radio"
#endif

#define BT_CORE_TYPE (bt_core_get_type())

typedef enum {
	BT_DEACTIVATED,
	BT_ACTIVATED,
	BT_ACTIVATING,
	BT_DEACTIVATING,
} bt_status_t;

typedef enum {
	BT_LE_DEACTIVATED,
	BT_LE_ACTIVATED,
	BT_LE_ACTIVATING,
	BT_LE_DEACTIVATING,
} bt_le_status_t;

typedef enum {
	BT_POWER_SAVING_MODE,
	BT_FLIGHT_MODE,
	BT_RECOVERY_MODE,
	BT_MODE_MAX
} bt_mode_e;

bt_status_t _bt_core_get_status(void);
bt_le_status_t _bt_core_get_le_status(void);
int _bt_core_get_bt_status(bt_mode_e mode);			/* Get the status of BT before passed mode */
int _bt_core_get_bt_le_status(bt_mode_e mode);			/* Get the status of BT LE before passed mode */
void _bt_core_set_bt_status(bt_mode_e mode, int status);	/* Set the status of BT before passed mode */
void _bt_core_set_bt_le_status(bt_mode_e mode, int status);	/* Set the status of BT LE before passed mode */

gboolean _bt_core_is_recovery_mode(void);
gboolean _bt_core_is_flight_mode_enabled(void);

int _bt_enable_adapter(void);
int _bt_disable_adapter(void);
int _bt_enable_adapter_le(void);
int _bt_disable_adapter_le(void);
int _bt_core_service_request_adapter(int service_function);

void _bt_core_adapter_added_cb(void);
void _bt_core_adapter_removed_cb(void);

gboolean _bt_core_enable_adapter(void);
gboolean _bt_core_disable_adapter(void);
gboolean _bt_core_recover_adapter(void);
gboolean _bt_core_enable_adapter_le(void);
gboolean _bt_core_disable_adapter_le(void);
gboolean __bt_core_reset_adapter(void);
gboolean _bt_core_enable_core(void);
gboolean _bt_core_set_transfer_value(gboolean value);
gboolean _bt_core_factory_test_mode(const char *type, const char *arg);

void _bt_core_update_status(void);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /*_BT_CORE_ADAPTER_H_*/
