/*
 * Open Adaptation Layer (OAL)
 *
 * Copyright (c) 2014-2015 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *              http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <stdint.h>

#ifndef _OAL_EVENT_H_
#define _OAL_EVENT_H_

#include <glib.h>
#include <sys/types.h>

#include <oal-manager.h>
#include <oal-gatt.h>

#ifdef __cplusplus
extern "C" {
#endif

#define FOREACH_EVENT(EVENT) \
	EVENT(OAL_EVENT_ADAPTER_ENABLED)	\
	EVENT(OAL_EVENT_ADAPTER_DISABLED)	\
	EVENT(OAL_EVENT_ADAPTER_HARDWARE_ERROR) \
	EVENT(OAL_EVENT_ADAPTER_PROPERTY_ADDRESS)			/* bt_address_t */	\
	EVENT(OAL_EVENT_ADAPTER_PROPERTY_NAME)				/* char string */\
	EVENT(OAL_EVENT_ADAPTER_PROPERTY_VERSION)			/* char string */\
	EVENT(OAL_EVENT_ADAPTER_PROPERTY_SERVICES)			/* event_adapter_services_t */\
	EVENT(OAL_EVENT_ADAPTER_MODE_NON_CONNECTABLE)	\
	EVENT(OAL_EVENT_ADAPTER_MODE_CONNECTABLE)	\
	EVENT(OAL_EVENT_ADAPTER_MODE_DISCOVERABLE)	\
	EVENT(OAL_EVENT_ADAPTER_MODE_DISCOVERABLE_TIMEOUT)	/* uint32_t */\
	EVENT(OAL_EVENT_ADAPTER_BONDED_DEVICE_LIST)		/* event_device_list_t */\
	EVENT(OAL_EVENT_ADAPTER_INQUIRY_STARTED)	\
	EVENT(OAL_EVENT_ADAPTER_INQUIRY_RESULT_BREDR_ONLY)	/* event_dev_found_t */\
	EVENT(OAL_EVENT_ADAPTER_INQUIRY_RESULT_BLE)		/* event_ble_dev_found_t */\
	EVENT(OAL_EVENT_ADAPTER_INQUIRY_FINISHED)	\
	EVENT(OAL_EVENT_DEVICE_PROPERTIES)					/* remote_device_t */\
	EVENT(OAL_EVENT_DEVICE_NAME)						/* remote_device_t */\
	EVENT(OAL_EVENT_DEVICE_VIDPID)						/* remote_device_t */\
	EVENT(OAL_EVENT_DEVICE_SERVICES)					/* event_dev_services_t */\
	EVENT(OAL_EVENT_DEVICE_PIN_REQUEST)				/* remote_device_t */\
	EVENT(OAL_EVENT_DEVICE_PASSKEY_ENTRY_REQUEST)		/* remote_device_t */\
	EVENT(OAL_EVENT_DEVICE_PASSKEY_CONFIRMATION_REQUEST)/* event_dev_passkey_t */\
	EVENT(OAL_EVENT_DEVICE_PASSKEY_DISPLAY)			/* event_dev_passkey_t */\
	EVENT(OAL_EVENT_DEVICE_SSP_CONSENT_REQUEST)		/* remote_device_t */\
	EVENT(OAL_EVENT_DEVICE_BONDING_SUCCESS)			/* bt_address_t */\
	EVENT(OAL_EVENT_DEVICE_BONDING_REMOVED)			/* bt_address_t */\
	EVENT(OAL_EVENT_DEVICE_BONDING_FAILED)				/* event_dev_bond_failed_t */\
	EVENT(OAL_EVENT_DEVICE_AUTHORIZE_REQUEST)			/* event_dev_authorize_req_t */\
	EVENT(OAL_EVENT_SOCKET_AUTHORIZE_REQUEST)			/* event_socket_authorize_req_t */\
	EVENT(OAL_EVENT_DEVICE_ACL_CONNECTED)				/* bt_address_t */\
	EVENT(OAL_EVENT_DEVICE_ACL_DISCONNECTED)			/* bt_address_t */\
	EVENT(OAL_EVENT_DEVICE_LE_CONNECTED)				/* bt_address_t */\
	EVENT(OAL_EVENT_DEVICE_LE_DISCONNECTED)			/* bt_address_t */\
	EVENT(OAL_EVENT_DEVICE_TRUSTED)			/* Remote Device is Trusted */\
	EVENT(OAL_EVENT_DEVICE_UNTRUSTED)			/* Remote Device is UnTrusted */\
	EVENT(OAL_EVENT_DEVICE_TRUSTED_PROFILES_CHANGED)		/* event_device_trusted_profiles_t */\
	EVENT(OAL_EVENT_RSSI_MONITORING_ENABLED)	/* event_dev_rssi_info_t */\
	EVENT(OAL_EVENT_RSSI_MONITORING_DISABLED)	/* event_dev_rssi_info_t */\
	EVENT(OAL_EVENT_RSSI_ALERT_RECEIVED)		/* event_dev_rssi_info_t */\
	EVENT(OAL_EVENT_RAW_RSSI_RECEIVED)		/* */\
	EVENT(OAL_EVENT_OAL_INITIALISED_SUCCESS)		/* OAL Initialisation event */	\
	EVENT(OAL_EVENT_OAL_INITIALISED_FAILED)			/* OAL Initialisation event */	\
	EVENT(OAL_EVENT_HID_CONNECTED)						/* event_hid_conn_t */\
	EVENT(OAL_EVENT_HID_DISCONNECTED)					/* event_hid_conn_t */\
	EVENT(OAL_EVENT_SOCKET_OUTGOING_CONNECTED)                 /* RFCOMM */  \
	EVENT(OAL_EVENT_SOCKET_DISCONNECTED)            /* RFCOMM */  \
	EVENT(OAL_EVENT_AUDIO_CONNECTING)                   /* bt_address_t */ \
	EVENT(OAL_EVENT_AUDIO_DISCONNECTING)                /* bt_address_t*/ \
	EVENT(OAL_EVENT_AUDIO_CONNECTED)                                        /* bt_address_t*/ \
	EVENT(OAL_EVENT_AUDIO_DISCONNECTED)                             /* bt_address_t*/  \
	EVENT(OAL_EVENT_AUDIO_STOPPED)                                          /* bt_address_t*/  \
	EVENT(OAL_EVENT_AUDIO_STARTED)                                          /* bt_address_t*/  \
	EVENT(OAL_EVENT_A2DP_SINK_DISCONNECTED)                                 /* bt_address_t*/  \
	EVENT(OAL_EVENT_A2DP_SINK_CONNECTED)                                    /* bt_address_t*/  \
	EVENT(OAL_EVENT_A2DP_SINK_DISCONNECTING)                                /* bt_address_t*/  \
	EVENT(OAL_EVENT_A2DP_SINK_CONNECTING)					/* bt_address_t*/ \
	EVENT(OAL_EVENT_HFP_DISCONNECTED)                /* bt_address_t*/ \
	EVENT(OAL_EVENT_HFP_CONNECTING)                /* bt_address_t*/ \
	EVENT(OAL_EVENT_HFP_CONNECTED)                /* bt_address_t*/ \
	EVENT(OAL_EVENT_HFP_DISCONNECTING)                /* bt_address_t*/ \
	EVENT(OAL_EVENT_HFP_AUDIO_DISCONNECTED)                /* bt_address_t*/ \
	EVENT(OAL_EVENT_HFP_AUDIO_CONNECTING)                /* bt_address_t*/ \
	EVENT(OAL_EVENT_HFP_AUDIO_CONNECTED)                /* bt_address_t*/ \
	EVENT(OAL_EVENT_HFP_AUDIO_DISCONNECTING)                /* bt_address_t*/ \
	EVENT(OAL_EVENT_AVRCP_CONNECTED)			/* AVRCP Target connected */ \
	EVENT(OAL_EVENT_AVRCP_DISCONNECTED)			/* AVRCP Target disconnected */ \
	EVENT(OAL_EVENT_AVRCP_REMOTE_FEATURES)                  /* AVRCP FATURE INFO*/  \
	EVENT(OAL_EVENT_AVRCP_SETTING_EQUALIZER_STATUS)         /* bt_address_t*/  \
	EVENT(OAL_EVENT_AVRCP_SETTING_REPEAT_STATUS)            /* bt_address_t*/  \
	EVENT(OAL_EVENT_AVRCP_SETTING_SHUFFLE_STATUS)           /* bt_address_t*/  \
	EVENT(OAL_EVENT_AVRCP_SETTING_SCAN_STATUS)                      /* bt_address_t*/  \
	EVENT(OAL_EVENT_AVRCP_VOLUME_MUTE_CHANGED)\
	EVENT(OAL_EVENT_AVRCP_DELAY_CHANGED)\
	EVENT(OAL_EVENT_AVRCP_CT_CONNECTED)                   /* bt_address_t*/ \
	EVENT(OAL_EVENT_AVRCP_CT_DISCONNECTED)                /* bt_address_t*/ \
	EVENT(OAL_EVENT_AVRCP_CT_EQUALIZER_STATUS)            /* property_value*/ \
	EVENT(OAL_EVENT_AVRCP_CT_REPEAT_STATUS)               /* property_value*/ \
	EVENT(OAL_EVENT_AVRCP_CT_SHUFFLE_STATUS)              /* property_value*/ \
	EVENT(OAL_EVENT_AVRCP_CT_SCAN_STATUS)                 /* property_value*/ \
	EVENT(OAL_EVENT_AVRCP_CT_NOTIF_PLAY_STATUS_CHANGED)   /* property_value*/ \
	EVENT(OAL_EVENT_AVRCP_CT_NOTIF_PLAY_POS_CHANGED)         /* property_value*/ \
	EVENT(OAL_EVENT_AVRCP_CT_NOTIF_TRACK_CHANGE)          /* trak_info*/ \
	EVENT(OAL_EVENT_AVRCP_CT_MEDIA_INFO)			/* trak_info response*/ \
	EVENT(OAL_EVENT_AVRCP_CT_PLAY_STATUS)						\
	EVENT(OAL_EVENT_AVRCP_CT_PASS_CMD_RES)		/* pass through command response*/ \
	EVENT(OAL_EVENT_AVRCP_CT_PLAYER_SETTING_RES)		/* palyer setting response*/ \
	EVENT(OAL_EVENT_AVRCP_CT_GET_PROPERTY_RES)            /* get propery response*/ \
	EVENT(OAL_EVENT_HDP_APP_REGISTERED)			/* HDP APP registered */ \
	EVENT(OAL_EVENT_HDP_APP_UNREGISTERED)			/* HDP APP un-registered */ \
	EVENT(OAL_EVENT_HDP_CHANNEL_CONNECTED)			/* HDP channel connected */ \
	EVENT(OAL_EVENT_HDP_CHANNEL_DESTROYED)			/* HDP channel destroyed */ \
	EVENT(OAL_EVENT_BLE_SERVER_INSTANCE_INITIALISED)    /*event_ble_adv_status */ \
		EVENT(OAL_EVENT_BLE_ENABLED)			/*ble adapter enable */ \
		EVENT(OAL_EVENT_BLE_DISABLED)			/*ble adapter disable */ \
	EVENT(OAL_EVENT_BLE_ADVERTISING_STARTED)                        /* NULL */\
	EVENT(OAL_EVENT_BLE_ADVERTISING_STOPPED)                        /* NULL*/\
	EVENT(OAL_EVENT_BLE_MULTI_ADVERTISING_ENABLE)           /* NULL*/\
	EVENT(OAL_EVENT_BLE_MULTI_ADVERTISING_DISABLE)          /* NULL*/\
	EVENT(OAL_EVENT_BLE_MULTI_ADVERTISING_SET_INST_DATA)            /* NULL*/\
	EVENT(OAL_EVENT_BLE_MULTI_ADVERTISING_UPDATE)           /* NULL*/\
	EVENT(OAL_EVENT_BLE_DISCOVERY_STARTED)                          /* NULL */\
	EVENT(OAL_EVENT_BLE_DISCOVERY_STOPPED)                          /* NULL */\
	EVENT(OAL_EVENT_BLE_REMOTE_DEVICE_FOUND)                /* event_ble_scan_result_info */\
	EVENT(OAL_EVENT_BLE_LOCAL_FEATURES)			/* adapter le features */\
	EVENT(OAL_EVENT_GATTS_SERVICE_ADDED)                /* gatts Service Added */\
	EVENT(OAL_EVENT_GATTS_INCLUDED_SERVICE_ADDED)       /* gatts Included Service Added */\
	EVENT(OAL_EVENT_GATTS_SERVICE_STARTED)              /* gatts Service Started */\
	EVENT(OAL_EVENT_GATTS_SERVICE_STOPED)               /* gatts Service Stoped */\
	EVENT(OAL_EVENT_GATTS_SERVICE_DELETED)              /* gatts Service Deleted  */\
	EVENT(OAL_EVENT_GATTS_CHARACTERISTIC_ADDED)         /* gatts Characteristic Added*/\
	EVENT(OAL_EVENT_GATTS_DESCRIPTOR_ADDED)             /* gatts Descriptor added */\
	EVENT(OAL_EVENT_GATTS_CONNECTION_COMPLETED)	    /* gatts Connection completed */\
	EVENT(OAL_EVENT_GATTS_DISCONNECTION_COMPLETED)	    /* gatts DisConnection completed */\
	EVENT(OAL_EVENT_GATTS_REQUEST_READ)                 /* gatts Request Read from client */\
	EVENT(OAL_EVENT_GATTS_REQUEST_WRITE)                /* gatts Request Write from client */\
	EVENT(OAL_EVENT_GATTS_RESPONSE_CONFIRMED)           /* gatts Response confirmation */\
	EVENT(OAL_EVENT_GATTS_IND_CONFIRM)                  /* gatts Indiction confirmation from remote client */\
	EVENT(OAL_EVENT_GATTS_NOTIFICATION)                 /* gatts Notification from remote client */\
	EVENT(OAL_EVENT_GATTS_MTU_CHANGED)                  /* gatts MTU changed */\
	EVENT(OAL_EVENT_GATTC_REGISTRATION)	/* gattc Registration Confirmed */\
	EVENT(OAL_EVENT_GATTC_CONNECTION_COMPLETED)     /* gattc connection completed */\
	EVENT(OAL_EVENT_GATTC_DISCONNECTION_COMPLETED)  /* gattc disconnection */\
	EVENT(OAL_EVENT_GATTC_NOTIFICATION_REGISTERED)  /* gattc notification register */\
	EVENT(OAL_EVENT_GATTC_NOTIFICATION_DEREGISTERED)        /* gattc notification deregister */\
	EVENT(OAL_EVENT_GATTC_SERVICE_SEARCH_RESULT)    /* gattc service search result info */\
	EVENT(OAL_EVENT_GATTC_SERVICE_SEARCH_DONE)      /* gattc service search complete */\
	EVENT(OAL_EVENT_GATTC_CHARAC_SERACH_RESULT)     /* gattc characteristic search result info */\
	EVENT(OAL_EVENT_GATTC_DESC_SERACH_RESULT)       /* gattc descriptor search result info */\
	EVENT(OAL_EVENT_GATTC_INCL_SRVC_SERACH_RESULT)  /* gattc included service search result info */\
	EVENT(OAL_EVENT_GATTC_REMOTE_RSSI)      /* remote device rssi */\
	EVENT(OAL_EVENT_GATTC_NOTIFY_DATA)      /* gattc notify data */\
	EVENT(OAL_EVENT_GATTC_READ_CHARAC)      /* gattc read characteristics */\
	EVENT(OAL_EVENT_GATTC_WRITE_CHARAC)     /* gattc write characteristics */\
	EVENT(OAL_EVENT_GATTC_READ_DESCR)       /* gattc read descriptor */\
	EVENT(OAL_EVENT_GATTC_WRITE_DESCR)      /* gattc write descriptor */\
	EVENT(OAL_EVENT_GATTC_MTU_EXCHANGE_COMPLETED)   /* gattc mtu configure complete */\
	EVENT(OAL_EVENT_GATTC_SERVICE_CHANGED_IND)      /* gattc service changed indication */\
	EVENT(OAL_EVENT_GATTC_PREPARE_WRITE)    /* gattc prepare write */\
	EVENT(OAL_EVENT_GATTC_EXECUTE_WRITE)    /* gattc execute write */\
	EVENT(OAL_EVENT_GATTS_REQUEST_ACQUIRE_WRITE)    /* gattc acquire write */\
	EVENT(OAL_EVENT_GATTS_REQUEST_ACQUIRE_NOTIFY)    /* gattc acquire notify */\
	EVENT(OAL_EVENT_END)                                /* End of event*/\


#define GENERATE_ENUM(ENUM) ENUM,
#define GENERATE_STRING(STRING) #STRING,

typedef enum EVENT_ENUM {
	FOREACH_EVENT(GENERATE_ENUM)
} oal_event_t;

#ifdef _OAL_EVENT_DISPATCHER_C_
const char *str_event[] = {
	FOREACH_EVENT(GENERATE_STRING)
};
#else
extern const char *str_event[];
#endif

/*********Datastructures for Local Adapter events ******************/
typedef struct {
	int num;
	bt_address_t devices[0];
} event_device_list_t;

typedef struct {
	remote_device_t device_info;
	uint8_t adv_data[62];
	int adv_len;
} event_ble_dev_found_t;

typedef struct {
	remote_device_t device_info;
} event_dev_found_t;

typedef struct {
	int num;
	service_uuid_t service_list[0];
} event_adapter_services_t;

typedef struct {
	/* TODO Add more features */
	uint8_t  max_adv_instance;
	uint8_t rpa_offloading;
	uint8_t max_adv_filter;
	uint8_t le_2m_phy_support;
	uint8_t le_coded_phy_support;
} event_adapter_le_features_t;

typedef struct {
	remote_device_t device_info;
	uint8_t adv_data[62];
	int adv_len;
} event_dev_properties_t;

/*********Datastructures for Remote Device events ******************/
typedef struct {
	bt_address_t address;
	int num;
	service_uuid_t service_list[0];
} event_dev_services_t;

typedef struct {
	remote_device_t device_info;
	uint32_t pass_key;
} event_dev_passkey_t;

typedef struct {
	bt_address_t address;
	oal_service_t service_id;
} event_dev_authorize_req_t;

typedef struct {
	bt_address_t address;
	oal_uuid_t uuid;
} event_socket_authorize_req_t;

typedef struct {
	bt_address_t address;
	oal_status_t status;
} event_dev_trust_t;

typedef struct {
	bt_address_t address;
	oal_status_t status;
} event_dev_conn_status_t;

typedef event_dev_conn_status_t event_dev_bond_failed_t;

typedef struct {
	bt_address_t address;
	unsigned int trust_val;
} event_device_trusted_profiles_t;

typedef struct {
	bt_address_t address;
	int link_type;
	int alert_type;
	int rssi;
} event_dev_rssi_info_t;

/*********Datastructures for HID callback******************/
/* HID :: connection state callback response data */
typedef struct {
	bt_address_t address;
	oal_status_t status;
} event_hid_conn_t;

/********* Datastructures for Socket event ******************/
/* SOCKET:: socket outgoing client connection event data */
typedef struct {
	int fd;			/**< FD of Outgoing client */
	int sock_type;		/**< Type of socket */
	bt_address_t address;	/**< Address of remote server */
} event_socket_client_conn_t;

/*********Datastructures for HDP callback******************/
/* HDP: App registration event data */
typedef struct {
	int app_id;
	oal_status_t status;
} event_hdp_app_reg_t;

/* HDP: Channel connection event data */
typedef struct {
	int fd;
	int app_id;
	int channel_id;
	int ch_type;
	bt_address_t address;
} event_hdp_channel_conn_t;

typedef struct {
	int server_inst;
	oal_uuid_t server_uuid;
} event_gatts_register_t;

typedef struct {
	int conn_id;
	int mtu_size;
} event_gatts_mtu_changed_t;

typedef struct {
	int server_inst;
} event_ble_adv_status;

typedef struct {
	int server_inst;
	int status;
} event_ble_multiadv_status;

/*GATT:: connection state callback response data*/
typedef struct {
	int server_inst;
	int conn_id;
	bt_address_t address;
	int status;
} event_gatts_conn_t;

typedef struct {
	int server_inst;
	int servic_hndl;
	oal_status_t status;
} event_gatts_srvc_t;

typedef struct {
	event_gatts_srvc_t gatt_srvc_stat;
	oal_gatt_srvc_id_t gatt_srvc_id;
} event_gatts_srvc_prm_t;

typedef struct {
	event_gatts_srvc_t gatt_srvc_stat;
	int incl_srvc_hndl;
} event_gatts_incld_srvc_t;

typedef struct {
	event_gatts_srvc_t gatt_srvc_stat;
	int descrptr_hndl;
	oal_uuid_t descrptr_uuid;
} event_gatts_srvc_descr_t;

typedef struct {
	event_gatts_srvc_t gatt_srvc_stat;
	int charctr_hndl;
	oal_uuid_t charctr_uuid;
} event_gatts_srvc_charctr_t;

typedef struct {
	int conn_id;
	int trans_id;
	int attr_handle;
	int offset;
} gatts_trans_attr_t;

typedef struct {
	gatts_trans_attr_t attr_trans;
	bt_address_t address;
	uint8_t is_long;
} event_gatts_srvc_read_attr_t;

typedef struct {
	gatts_trans_attr_t attr_trans;
	int mtu;
	char *path;
	bt_address_t address;
} event_gatts_srvc_acquire_attr_t  ;

typedef struct {
	gatts_trans_attr_t attr_trans;
	int length;
	bt_address_t address;
	uint8_t value[OAL_GATT_MAX_ATTR_LEN];
	uint8_t need_rsp;
	uint8_t is_prep;
} event_gatts_srvc_write_attr_t;

typedef struct {
	int hndl;
	oal_status_t status;
} event_gatts_respons_t;

typedef struct {
	int conn_id;
	int trans_id;
	int attr_handle;
	bt_address_t address;
} event_gatts_ind_cnfrm_t;

typedef struct {
	int conn_id;
	int trans_id;
	int attr_handle;
	uint8_t notify;
	bt_address_t address;
} event_gatts_notif_t;

/* BLE: Scan Result Callback data */
typedef struct {
	int rssi;
	bt_address_t address;
	uint8_t adv_data[BLE_ADV_DATA_LENGTH];
} event_ble_scan_result_info;

typedef struct {
	oal_status_t status;
	int client_if;
	oal_uuid_t client_uuid;
} event_gattc_register_t;

typedef struct {
	oal_status_t status;
	int client_if;
	int conn_id;
	bt_address_t address;
} event_gattc_conn_t;

typedef struct {
	oal_status_t status;
	int conn_id;
} event_gattc_conn_status_t;

typedef struct {
	event_gattc_conn_status_t conn_status;
	oal_gatt_srvc_id_t srvc_id;
} event_gattc_service_result_t;

typedef struct {
	int char_prop;
	event_gattc_conn_status_t conn_status;
	oal_gatt_id_t char_id;
	oal_gatt_srvc_id_t srvc_id;
} event_gattc_characteristic_result_t;

typedef struct {
	event_gattc_conn_status_t conn_status;
	oal_gatt_id_t char_id;
	oal_gatt_id_t descr_id;
	oal_gatt_srvc_id_t srvc_id;
} event_gattc_descriptor_result_t;

typedef struct {
	oal_status_t status;
	int client_if;
	oal_gatt_id_t char_id;
	oal_gatt_srvc_id_t srvc_id;
} event_gattc_regdereg_notify_t;


#define BLE_READ_NOTIFY_DATA_LENGTH     600

typedef struct {
	int is_notify;
	uint16_t data_len;
	uint8_t data[BLE_READ_NOTIFY_DATA_LENGTH];
	bt_address_t address;
	oal_gatt_id_t char_id;
	oal_gatt_srvc_id_t srvc_id;
} event_gattc_notify_data;

typedef struct {
	uint16_t value_type;
	uint16_t data_len;
	uint8_t data[BLE_READ_NOTIFY_DATA_LENGTH];
	event_gattc_descriptor_result_t uuid_status;
} event_gattc_read_data;

typedef event_gattc_descriptor_result_t event_gattc_write_data;

/********* AVRCP CT role callback data structures ******************/
#define MEDIA_ATTIRBUTE_STRING_LENGTH 100
typedef struct {
	uint8_t title_info[MEDIA_ATTIRBUTE_STRING_LENGTH];
	uint8_t artist_info[MEDIA_ATTIRBUTE_STRING_LENGTH];
	uint8_t album_info[MEDIA_ATTIRBUTE_STRING_LENGTH];
	uint64_t playing_time;
	uint8_t genre_info[MEDIA_ATTIRBUTE_STRING_LENGTH];
	uint64_t total_track;
	uint64_t track_number;
} event_avrcp_ct_media_info_t;

typedef enum {
	OAL_PLAY_STATUS_STOPPED = 0x00,
	OAL_PLAY_STATUS_PLAYING = 0x01,
	OAL_PLAY_STATUS_PAUSED = 0x02,
	OAL_PLAY_STATUS_FWD_SEEK = 0x03,
	OAL_PLAY_STATUS_REV_SEEK = 0x04,
	OAL_PLAY_STATUS_ERROR = 0xFF,
} notif_event_avrcp_ct_play_status_t;

#define BTRCC_UID_SIZE  8
typedef uint8_t notif_event_avrcp_ct_track_id[BTRCC_UID_SIZE];

typedef uint32_t notif_event_avrcp_ct_song_pos;

typedef union {
	notif_event_avrcp_ct_play_status_t play_status;
	notif_event_avrcp_ct_track_id track_id;
	notif_event_avrcp_ct_song_pos song_pos;
} event_notif_avrcp_ct_notif_info_t;

typedef struct {
	uint8_t song_len;
	uint8_t song_pos;
	notif_event_avrcp_ct_play_status_t play_status;
} event_avrcp_ct_play_status_t;

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /*_OAL_EVENT_H_*/
