/*
 * Copyright (c) 2016 - 2017 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact:  Anupam Roy <anupam.r@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *		http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */


#ifndef _BT_SERVICE_CORE_ADAPTER_LE_H_
#define _BT_SERVICE_CORE_ADAPTER_LE_H_

#include <glib.h>
#include <sys/types.h>
#include "bluetooth-api.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
	LE_DISCOVERY_STOPPED,
	LE_DISCOVERY_STARTED,
	LE_DISCOVERY_STARTING,
	LE_DISCOVERY_STOPPING,
} bt_le_discovery_state_t;

typedef enum {
	BT_LE_DEACTIVATED,
	BT_LE_ACTIVATED,
	BT_LE_ACTIVATING,
	BT_LE_DEACTIVATING,
} bt_le_status_t;

int _bt_enable_adapter_le(void);

int _bt_disable_adapter_le(void);

int _bt_le_init(void);

void _bt_le_deinit(void);

int _bt_is_advertising(void);
int _bt_set_advertising(const char *sender, int adv_handle, gboolean enable, gboolean use_reserved_slot);

int _bt_set_custom_advertising(const char *sender, int adv_handle, gboolean enable, bluetooth_advertising_params_t *params, gboolean use_reserved_slot);

int _bt_set_advertising_data(const char *sender, int adv_handle, bluetooth_advertising_data_t *data, int length, gboolean use_reserved_slot);

int _bt_set_scan_response_data(const char *sender, int adv_handle, bluetooth_scan_resp_data_t *response, int length, gboolean use_reserved_slot);

int _bt_get_advertising_data(char *sender, int adv_handle, bluetooth_advertising_data_t *adv, int *length);

int _bt_get_scan_response_data(char *sender, int adv_handle, bluetooth_scan_resp_data_t *adv, int *length);

int _bt_start_le_scan(const char *sender);

int _bt_stop_le_scan(const char *sender);

int _bt_set_scan_parameters(bluetooth_le_scan_params_t *params);

gboolean _bt_is_le_scanning(void);

void _bt_check_le_scanner_app_termination(const char *sender);

void _bt_adapter_set_le_status(bt_le_status_t status);

bt_le_status_t _bt_adapter_get_le_status(void);

int _bt_service_le_init(void);

void _bt_service_le_deinit(void);

gboolean _bt_is_le_2m_phy_supported(void);

gboolean _bt_is_le_coded_phy_supported(void);

int _bt_set_le_static_random_address(gboolean is_enable);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /*_BT_SERVICE_CORE_ADAPTER_LE_H_*/
