/*
 * Copyright (c) 2015 - 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Contact:  Anupam Roy <anupam.r@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *		http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */


#ifndef _BT_SERVICE_CORE_ADAPTER_H_
#define _BT_SERVICE_CORE_ADAPTER_H_

#include <sys/types.h>
#ifdef TIZEN_FEATURE_BT_OBEX
#include "alarm.h"
#endif
#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
	ADAPTER_DISCOVERY_STOPPED,
	ADAPTER_DISCOVERY_STARTED,
	ADAPTER_DISCOVERY_STARTING,
	ADAPTER_DISCOVERY_STOPPING,
} bt_adapter_discovery_state_t;

typedef enum {
	BT_DEACTIVATED,
	BT_ACTIVATED,
	BT_ACTIVATING,
	BT_DEACTIVATING,
} bt_status_t;

int _bt_enable_adapter(void);

int _bt_disable_adapter(void);

int _bt_reset_adapter(void);

int _bt_check_adapter(int *status);

int _bt_enable_core(void);

int _bt_recover_adapter(void);

int _bt_start_discovery(unsigned short max_response,
		unsigned short duration, unsigned int cod_mask);

int _bt_start_custom_discovery(bt_discovery_role_type_t role);

int _bt_cancel_discovery(void);

void _bt_service_register_vconf_handler(void);

void _bt_service_register_vconf_handler(void);

gboolean _bt_is_discovering(void);

int _bt_stack_init(void);

int _bt_get_local_address(void);

int _bt_get_local_version(void);

int _bt_get_local_name(void);

int _bt_set_local_name(char *local_name);

int _bt_get_discoverable_mode(int *mode);

int _bt_get_timeout_value(int *timeout);

int _bt_set_discoverable_mode(int discoverable_mode, int timeout);

gboolean _bt_is_connectable(void);

int _bt_is_service_used(void);

int _bt_set_connectable(gboolean connectable);

int _bt_adapter_get_bonded_devices(void);

int _bt_get_profile_connected_devices(char *profile_uuid, GArray **addr_list);

int _bt_init_profiles();

int _bt_cleanup_profiles(void);

void _bt_adapter_set_status(bt_status_t status);

bt_status_t _bt_adapter_get_status(void);

void _bt_set_disabled(int result);

int _bt_set_adapter_request_state(int enable);

int _bt_set_le_request_state(int enable);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /*_BT_SERVICE_CORE_ADAPTER_H_*/

