/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *		http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */


#ifndef _BT_SERVICE_UTIL_H_
#define _BT_SERVICE_UTIL_H_

#include <sys/types.h>
#include <gio/gio.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define BT_NODE_NAME_LEN 50
#define BT_UUID_LENGTH_MAX 16
#define BT_UUID_STRING_SIZE 37


static const char BT_SERVICE_BASE_UUID[] = {
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00,
	0x80, 0x00, 0x00, 0x80, 0x5f, 0x9b, 0x34, 0xfb
};

#if defined(TIZEN_FEATURE_BT_OBEX) || defined(TIZEN_FEATURE_BT_PAN_NAP)
typedef struct {
	int req_id;
	int service_function;
	char name[BT_NODE_NAME_LEN];
	GDBusMethodInvocation *context;
} request_info_t;

request_info_t *_bt_get_request_info(int req_id);

int _bt_delete_request_list(int req_id);

#endif

#ifdef TIZEN_FEATURE_BT_OBEX

void _bt_init_request_id(void);

int _bt_assign_request_id(void);

void _bt_delete_request_id(int request_id);


void _bt_init_request_list(void);

int _bt_insert_request_list(int req_id, int service_function,
			char *name, GDBusMethodInvocation *context);


void _bt_clear_request_list(void);
#endif

void _bt_service_convert_uuid_type_to_string(char *str, const unsigned char *uuid);
void _bt_service_convert_uuid_string_to_type(unsigned char *uuid, const char *str);

char * _bt_service_convert_hex_to_string(unsigned char *hex, int len);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /*_BT_SERVICE_UTIL_H_*/

