/*
 * Copyright (c) 2011 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *		http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */


#ifndef _BT_SERVICE_PROXIMITY_H_
#define _BT_SERVICE_PROXIMITY_H_

#include <glib.h>
#include <sys/types.h>
#include "bluetooth-api.h"

#ifdef __cplusplus
extern "C" {
#endif


typedef enum {
	BT_PXP_ALERT_NONE = 0,
	BT_PXP_ALERT_MILD,
	BT_PXP_ALERT_HIGH
} bt_proximity_alert_value_t;

typedef enum {
	BT_PXP_PROPERTY_LLS = 0x01,
	BT_PXP_PROPERTY_IAS = 0x02,
	BT_PXP_PROPERTY_TX_POWER = 0x04
} bt_proximity_alert_property_t;

typedef enum {
	BT_PXP_REPORTER_ROLE = 0x00,
	BT_PXP_MONITOR_ROLE = 0x01,
} bt_proximity_role_t;

int bt_set_proximity_property(bluetooth_device_address_t *device_address,
		unsigned int property, int value);

int bt_get_proximity_property(bluetooth_device_address_t *device_address,
		unsigned int property, int *value);

int bt_get_proximity_supported_services(bluetooth_device_address_t *device_address,
		unsigned int *supported_services);

int bt_register_proximity_reporter();

int bt_unregister_proximity_reporter();

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /*_BT_SERVICE_PROXIMITY_H_*/
