/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *		http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef __BLUETOOTH_PLUGIN_H
#define __BLUETOOTH_PLUGIN_H

#include <stdbool.h>
#include <dlog.h>


#undef LOG_TAG
#define LOG_TAG "BLUETOOTH_FRWK_PLUGIN"

#define LOG_COLOR_RESET    "\033[0m"
#define LOG_COLOR_RED      "\033[31m"
#define LOG_COLOR_YELLOW   "\033[33m"
#define LOG_COLOR_GREEN         "\033[32m"
#define LOG_COLOR_BLUE          "\033[36m"
#define LOG_COLOR_PURPLE   "\033[35m"

#define BT_DBG(fmt, args...) \
	((SLOGD(fmt, ##args)))
#define BT_INFO(fmt, args...) \
	((SLOGI(fmt, ##args)))
#define BT_ERR(fmt, args...) \
	((SLOGE(fmt, ##args)))

#define BT_INFO_C(fmt, arg...) \
	SLOGI_IF(TRUE,  LOG_COLOR_GREEN" "fmt" "LOG_COLOR_RESET, ##arg)
#define BT_ERR_C(fmt, arg...) \
	SLOGI_IF(TRUE,  LOG_COLOR_RED" "fmt" "LOG_COLOR_RESET, ##arg)

#define ret_if(expr) \
	do { \
		if (expr) { \
			BT_ERR("(%s) return", #expr); \
			return; \
		} \
	} while (0)

#define retv_if(expr, val) \
	do { \
		if (expr) { \
			BT_ERR("(%s) return", #expr); \
			return (val); \
		} \
	} while (0)

typedef enum {
	BT_AGENT_EVENT_PIN_REQUEST,
	BT_AGENT_EVENT_PASSKEY_CONFIRM_REQUEST,
	BT_AGENT_EVENT_PASSKEY_AUTO_ACCEPTED,
	BT_AGENT_EVENT_PASSKEY_REQUEST,
	BT_AGENT_EVENT_PASSKEY_DISPLAY_REQUEST,
	BT_AGENT_EVENT_AUTHORIZE_REQUEST,
	BT_AGENT_EVENT_CONFIRM_MODE_REQUEST,
	BT_AGENT_EVENT_APP_CONFIRM_REQUEST,
	BT_AGENT_EVENT_FILE_RECEIVED,
	BT_AGENT_EVENT_KEYBOARD_PASSKEY_REQUEST,
	BT_AGENT_EVENT_SECURITY,
	BT_AGENT_EVENT_TERMINATE,
	BT_AGENT_EVENT_EXCHANGE_REQUEST,
	BT_AGENT_EVENT_PBAP_REQUEST,
	BT_AGENT_EVENT_MAP_REQUEST,
	BT_AGENT_EVENT_SYSTEM_RESET_REQUEST,
	BT_AGENT_EVENT_LEGACY_PAIR_FAILED_FROM_REMOTE,
} bt_agent_event_type_t;

struct bluetooth_headed_plugin_t {
	int (*bt_launch_dpmpopup)(char *mode);
	int (*bt_launch_system_popup)(bt_agent_event_type_t event_type,
							const char *device_name,
							const unsigned char *auth_info,
							char *passkey,
							const char *filename,
							const char *agent_path);

	void (*bt_destroy_popup_all)(void);
	bool (*bt_launch_unable_to_pairing_syspopup)(int result);
	bool (*bt_is_tethering_enabled)(void);
	int (*bt_get_mime_type)(char *file_name, char **mime_type);
};

#ifdef TIZEN_FEATURE_BT_DPM
#define BT_DPM_SYSPOPUP "dpm-syspopup"
#endif



#endif /* __BLUETOOTH_PLUGIN_H */
